/*************************************************************************************
  Copyright (C) 2014 Laboratori de Gràfics i Imatge, Universitat de Girona &
  Institut de Diagnòstic per la Imatge.
  Girona 2014. All rights reserved.
  http://starviewer.udg.edu

  This file is part of the Starviewer (Medical Imaging Software) open source project.
  It is subject to the license terms in the LICENSE file found in the top-level
  directory of this distribution and at http://starviewer.udg.edu/license. No part of
  the Starviewer (Medical Imaging Software) open source project, including this file,
  may be copied, modified, propagated, or distributed except according to the
  terms contained in the LICENSE file.
 *************************************************************************************/

#include "standardizeduptakevaluebodyweightformulacalculator.h"

#include "dicomtagreader.h"

namespace udg {

StandardizedUptakeValueBodyWeightFormulaCalculator::StandardizedUptakeValueBodyWeightFormulaCalculator()
 : StandardizedUptakeValueFormulaCalculator()
{
    initializeNormalizationFactorParameter();
}

StandardizedUptakeValueBodyWeightFormulaCalculator::~StandardizedUptakeValueBodyWeightFormulaCalculator()
{
}

void StandardizedUptakeValueBodyWeightFormulaCalculator::initializeNormalizationFactorParameter()
{
    m_bodyWeightInGr = 0;
}

bool StandardizedUptakeValueBodyWeightFormulaCalculator::normalizationFactorValueIsValid() const
{
    return m_bodyWeightInGr > 0;
}

void StandardizedUptakeValueBodyWeightFormulaCalculator::gatherRequiredNormalizationFactorParameter()
{
    initializeNormalizationFactorParameter();
    gatherRequiredNormalizationFactorParameter(m_tagReaderSource);
}

void StandardizedUptakeValueBodyWeightFormulaCalculator::gatherRequiredNormalizationFactorParameter(DICOMTagReader *tagReader)
{
    if (!tagReader)
    {
        return;
    }

    m_bodyWeightInGr = tagReader->getValueAttributeAsQString(DICOMPatientWeight).toDouble() * 1000;
}

double StandardizedUptakeValueBodyWeightFormulaCalculator::getNormalizationFactor()
{
    return m_bodyWeightInGr;
}

QString StandardizedUptakeValueBodyWeightFormulaCalculator::getNormalizationFactorUnits() const
{
    return  "g";
}

} // End namespace udg
